from django.test import TestCase
from django.forms import PasswordInput

from tsuru_dashboard.auth.forms import ChangePasswordForm


class ChangePasswordFormTest(TestCase):
    def test_form_is_valid(self):
        data = {
            "old": "old",
            "new": "new",
            "confirm": "new",
        }
        form = ChangePasswordForm(data)
        self.assertTrue(form.is_valid())

    def test_old_is_required(self):
        data = {
            "new": "new",
            "confirm": "new",
        }
        form = ChangePasswordForm(data)
        self.assertFalse(form.is_valid())

    def test_new_is_required(self):
        data = {
            "old": "old",
            "confirm": "new",
        }
        form = ChangePasswordForm(data)
        self.assertFalse(form.is_valid())

    def test_confirm_is_required(self):
        data = {
            "old": "old",
            "new": "new",
        }
        form = ChangePasswordForm(data)
        self.assertFalse(form.is_valid())

    def test_old_use_password_input(self):
        old_field = ChangePasswordForm.base_fields['old']
        self.assertIsInstance(old_field.widget, PasswordInput)

    def test_new_use_password_input(self):
        new_field = ChangePasswordForm.base_fields['new']
        self.assertIsInstance(new_field.widget, PasswordInput)

    def test_confirm_use_password_input(self):
        confirm_field = ChangePasswordForm.base_fields['confirm']
        self.assertIsInstance(confirm_field.widget, PasswordInput)



from django.test import TestCase

from tsuru_dashboard import settings
from tsuru_dashboard.auth.forms import TokenRequestForm

from mock import patch, Mock


class TokenRequestFormTest(TestCase):

    def test_email_is_required(self):
        form = TokenRequestForm()
        self.assertFalse(form.is_valid())

        form = TokenRequestForm({"email": "a@a.com"})
        self.assertTrue(form.is_valid())

    def test_email_should_be_an_email(self):
        form = TokenRequestForm({"email": "a@a.com"})
        self.assertTrue(form.is_valid())

        invalid_emails = ["a", "a@a", "a@a."]

        for email in invalid_emails:
            form = TokenRequestForm({"email": email})
            self.assertFalse(form.is_valid())

    @patch("requests.post")
    @patch("requests.get")
    def test_send_request(self, get, post):
        get.return_value = Mock(status_code=200)
        form = TokenRequestForm({"email": "a@a.com"})
        form.is_valid()
        form.send()
        url = "{0}/users/{1}/password".format(settings.TSURU_HOST,
                                              form.cleaned_data["email"])
        post.assert_called_with(url)



import requests
import grequests
import json
from urlparse import urlparse
from dateutil import parser

from django.views.generic import TemplateView
from django.http import HttpResponse, Http404, JsonResponse, StreamingHttpResponse
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.contrib import messages

from pygments import highlight
from pygments.lexers import DiffLexer
from pygments.formatters import HtmlFormatter
from pytz import utc

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView


class PoolList(LoginRequiredView, TemplateView):
    template_name = "admin/pool_list.html"

    def extract_ip(self, address):
        if not urlparse(address).scheme:
            address = "http://"+address
        return urlparse(address).hostname

    def get_node(self, address, data):
        for response in data:
            if response.status_code != 200:
                continue

            node_units = response.json()
            if not node_units:
                continue

            if 'HostAddr' not in node_units[0]:
                continue

            if self.extract_ip(node_units[0]['HostAddr']) == self.extract_ip(address):
                return node_units
        return []

    def units_by_node(self, address, units):
        units = self.get_node(address, units)
        result = {}

        for unit in units:
            if not unit['Status'] in result:
                result[unit['Status']] = 0

            result[unit['Status']] += 1

        len_units = len(units)
        if len_units > 0:
            result['total'] = len_units

        return result

    def node_last_success(self, date):
        if date:
            last_success = parser.parse(date)
            if last_success.tzinfo:
                last_success = last_success.astimezone(utc)
            else:
                last_success = utc.localize(last_success)
            return last_success
        return date

    def nodes_by_pool(self):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        pools = {}

        if response.status_code != 204:
            data = response.json()
            nodes = data.get("nodes", [])

            url = "{}/docker/node/{}/containers"
            urls = [url.format(settings.TSURU_HOST, node["Address"]) for node in nodes]

            rs = (grequests.get(u, headers=self.authorization) for u in urls)
            units = grequests.map(rs)

            for node in nodes:
                dt = node["Metadata"].get("LastSuccess")
                node["Metadata"]["LastSuccess"] = self.node_last_success(dt)
                node["Units"] = self.units_by_node(node["Address"], units)
                pool = node["Metadata"].get("pool")
                nodes_by_pool = pools.get(pool, [])
                nodes_by_pool.append(node)
                pools[pool] = nodes_by_pool
        return sorted(pools.items())

    def get_context_data(self, *args, **kwargs):
        context = super(PoolList, self).get_context_data(*args, **kwargs)
        context.update({"pools": self.nodes_by_pool()})
        return context


class NodeInfo(LoginRequiredView, TemplateView):
    template_name = "admin/node_info.html"


class NodeInfoJson(LoginRequiredView):
    def get_containers(self, node_address):
        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, node_address)
        response = requests.get(url, headers=self.authorization)

        if response.status_code == 204:
            return []

        if response.status_code > 399:
            return []

        return response.json() or []

    def get_node(self, address):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)

        if response.status_code != 204:
            data = response.json()
            nodes = data.get("nodes", [])

            for node in nodes:
                if node["Address"] == address:
                    return node

        return None

    def get(self, *args, **kwargs):
        containers = self.get_containers(kwargs["address"])
        for container in containers:
            if "AppName" in container:
                container["DashboardURL"] = reverse(
                    'detail-app', kwargs={'app_name': container["AppName"]})
        return JsonResponse({
            "node": {
                "info": self.get_node(kwargs["address"]),
                "containers": containers,
                "nodeRemovalURL": reverse('node-remove', kwargs={'address': kwargs["address"]})
            }
        })


class ListDeploy(LoginRequiredView, TemplateView):
    template_name = "deploys/list_deploys.html"

    def get_context_data(self, *args, **kwargs):
        context = super(ListDeploy, self).get_context_data(*args, **kwargs)

        page = int(self.request.GET.get('page', '1'))

        skip = (page * 20) - 20
        limit = page * 20

        url = '{}/deploys?skip={}&limit={}'.format(settings.TSURU_HOST, skip, limit)

        response = requests.get(url, headers=self.authorization)

        if response.status_code == 204:
            deploys = []
        else:
            deploys = response.json()

        context['deploys'] = deploys

        if len(deploys) >= 20:
            context['next'] = page + 1

        if page > 0:
            context['previous'] = page - 1

        return context


class DeployInfo(LoginRequiredView, TemplateView):
    template_name = "deploys/deploy_details.html"

    def get_context_data(self, *args, **kwargs):
        deploy_id = kwargs["deploy"]

        url = "{}/deploys/{}".format(settings.TSURU_HOST, deploy_id)
        response = requests.get(url, headers=self.authorization)

        if response.status_code > 399:
            raise Http404("Deploy does not exist")

        context = {"deploy": response.json()}

        diff = context["deploy"].get("Diff")
        if diff and diff != u'The deployment must have at least two commits for the diff.':
            format = HtmlFormatter()
            diff = highlight(diff, DiffLexer(), format)
        else:
            diff = None

        context["deploy"]["Diff"] = diff
        return context


class ListHealing(LoginRequiredView, TemplateView):
    template_name = "docker/list_healing.html"

    def get_context_data(self, *args, **kwargs):
        context = super(ListHealing, self).get_context_data(*args, **kwargs)
        url = '{}/docker/healing'.format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        formatted_events = []
        if response.status_code == 200:
            events = response.json() or []
            formatted_events = []

            for event in events:
                event['FailingContainer']['ID'] = event['FailingContainer']['ID'][:12]
                event['CreatedContainer']['ID'] = event['CreatedContainer']['ID'][:12]
                event['App'] = event['FailingContainer']['AppName']
                formatted_events.append(event)

        context.update({"events": formatted_events})
        return context


class PoolInfo(LoginRequiredView, TemplateView):
    template_name = "docker/pool_info.html"

    def get_node(self, address, data):
        for response in data:
            if response.status_code != 200:
                continue

            node_units = response.json()
            if not node_units:
                continue

            if 'HostAddr' not in node_units[0]:
                continue

            if node_units[0]['HostAddr'] in address:
                return node_units
        return []

    def units_by_node(self, address, units):
        units = self.get_node(address, units)
        result = {}

        for unit in units:
            if not unit['Status'] in result:
                result[unit['Status']] = 0

            result[unit['Status']] += 1

        len_units = len(units)
        if len_units > 0:
            result['total'] = len_units

        return result

    def node_last_success(self, date):
        if date:
            last_success = parser.parse(date)
            if last_success.tzinfo:
                last_success = last_success.astimezone(utc)
            else:
                last_success = utc.localize(last_success)
            return last_success
        return date

    def nodes_by_pool(self, pool):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        pools = {}

        if response.status_code != 204:
            data = response.json()
            nodes = data.get("nodes", [])

            url = "{}/docker/node/{}/containers"
            urls = [url.format(settings.TSURU_HOST, node["Address"]) for node in nodes]

            rs = (grequests.get(u, headers=self.authorization) for u in urls)
            units = grequests.map(rs)

            for node in nodes:
                if node["Metadata"].get("pool", "") != pool:
                    continue

                dt = node["Metadata"].get("LastSuccess")
                node["Metadata"]["LastSuccess"] = self.node_last_success(dt)

                node["Units"] = self.units_by_node(node["Address"], units)

                pool = node["Metadata"].get("pool")
                nodes_by_pool = pools.get(pool, [])
                nodes_by_pool.append(node)
                pools[pool] = nodes_by_pool

        return pools

    def get_context_data(self, *args, **kwargs):
        context = super(PoolInfo, self).get_context_data(*args, **kwargs)
        context.update({"pools": self.nodes_by_pool(kwargs["pool"])})
        return context


class NodeRemove(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        address = self.kwargs['address']

        msg = u"The value for '{}' parameter should be 'true' or 'false'"

        destroy = self.request.GET.get("destroy", "false")
        if destroy not in ["true", "false"]:
            return HttpResponse(msg.format("destroy"), status=400)

        rebalance = self.request.GET.get("rebalance", "false")
        if rebalance not in ["true", "false"]:
            return HttpResponse(msg.format("rebalance"), status=400)
        no_rebalance = "false" if rebalance == "true" else "true"

        response = requests.delete(
            '{}/docker/node/{}?remove-iaas={}&no-rebalance={}'.format(
                settings.TSURU_HOST, address, destroy, no_rebalance
            ),
            headers=self.authorization
        )

        if response.status_code > 399:
            return HttpResponse(response.text, status=response.status_code)

        return redirect(reverse('pool-list'))


class TemplateListJson(LoginRequiredView):

    def get(self, *args, **kwargs):
        templates = self.client.templates.list()
        return JsonResponse(templates, safe=False)


class NodeAdd(LoginRequiredView):

    def post(self, *args, **kwargs):
        resp = self.client.nodes.create(**self.request.POST.dict())

        for line in resp.iter_lines():
            msg_err = json.loads(line).get('Error')
            if msg_err:
                messages.error(self.request, msg_err, fail_silently=True)
                return HttpResponse(msg_err, status=500)

        messages.success(self.request, u'Node was successfully created', fail_silently=True)
        return HttpResponse('Node was successfully created', status=200)


class PoolRebalance(LoginRequiredView):

    def post(self, *args, **kwargs):
        def sending_stream():
            r = self.client.pools.rebalance(pool=kwargs["pool"])
            for line in r.iter_lines():
                yield "{}<br>".format(line)
        return StreamingHttpResponse(sending_stream())



from django.test import TestCase
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.forms import PasswordRecoveryForm

from mock import patch, Mock


class PasswordRecoveryViewTest(TestCase):
    def test_get(self):
        response = self.client.get(reverse('password-recovery'))
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed('auth/password_recovery.html')
        self.assertIsInstance(response.context['form'], PasswordRecoveryForm)

    @patch("requests.post")
    @patch("requests.get")
    def test_send_when_form_is_valid(self, get, post):
        get.return_value = Mock(status_code=200)
        response = self.client.post(reverse('password-recovery'),
                                    {"email": "some@email.com", "token": "tt"})
        self.assertRedirects(response, reverse('password-recovery-success'))
        url = "{0}/users/{1}/password?token={2}".format(
            settings.TSURU_HOST,
            "some@email.com",
            "tt"
        )
        post.assert_called_with(url)



from django.test import TestCase

from tsuru_dashboard import settings
from tsuru_dashboard.auth.forms import PasswordRecoveryForm

from mock import patch


class PasswordRecoveryFormTest(TestCase):
    def test_email_is_required(self):
        form = PasswordRecoveryForm({"token": "token"})
        self.assertFalse(form.is_valid())

    def test_token_is_required(self):
        form = PasswordRecoveryForm({"email": "a@a.com"})
        self.assertFalse(form.is_valid())

    def test_email_should_be_an_email(self):
        invalid_emails = ["a", "a@a", "a@a."]

        for email in invalid_emails:
            form = PasswordRecoveryForm({"email": email, "token": "token"})
            self.assertFalse(form.is_valid())

    def test_valid(self):
        form = PasswordRecoveryForm({"email": "a@a.com", "token": "token"})
        self.assertTrue(form.is_valid())

    @patch("requests.post")
    def test_send(self, post):
        form = PasswordRecoveryForm({"email": "a@a.com", "token": "token"})
        form.is_valid()
        form.send()
        url = "{0}/users/{1}/password?token={2}".format(
            settings.TSURU_HOST,
            form.cleaned_data["email"],
            form.cleaned_data["token"]
        )
        post.assert_called_with(url)



# coding: utf-8

import os

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

DEBUG = os.environ.get("DEBUG", "false") in ("true", "True", "1")

SECRET_KEY = os.environ.get("SECRET_KEY", "your-secret-key")

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'data.db'),
    }
}

TIME_ZONE = os.environ.get('TSURU_DASHBOARD_TIME_ZONE', 'America/Sao_Paulo')

LANGUAGE_CODE = 'en-us'

USE_I18N = True

USE_L10N = True

USE_TZ = True

STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                "django.template.context_processors.static",
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'abyss.urls'

WSGI_APPLICATION = 'abyss.wsgi.application'

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'tsuru_dashboard',
]

SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'filters': ['require_debug_false'],
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        }
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
        'dashboard': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': True,
        },
    }
}

ALLOWED_HOSTS = ["*"]

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'tsuru_cache',
    }
}

RVN_CONFIG = os.environ.get("RAVEN_CONFIG")
if RVN_CONFIG:
    RAVEN_CONFIG = {
        'dsn': RVN_CONFIG,
    }

    INSTALLED_APPS = INSTALLED_APPS + [
        'raven.contrib.django.raven_compat',
    ]



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.apps.views import ListApp


class ListAppViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_list_template(self, token_is_valid):
        token_is_valid.return_value = True
        response = ListApp.as_view()(self.request)
        self.assertIn("apps/list.html", response.template_name)
